#!/usr/bin/env bash

#    Copyright (C) 2011  Pablo Fernandez Fernandez <pfernandez 13t org>
#
#    GridBob is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    GridBob is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with GridBob.  If not, see <http://www.gnu.org/licenses/>.

MIN=$1
MAX=$2
let SIZE=MAX-MIN

# We don't pass MAX as a parameter!!
# We prepare ourselves in case the number is bigger than a 31bit integer to avoid sign.
#  so we calculate the number it turns around 2.147.483.648
let TURNS_MIN=MIN/2147483648
let MIN_MODIF=MIN%2147483648
let TURNS_MAX=MAX/2147483648
let MAX_MODIF=MAX%2147483648

# Start printing status
echo "Grid John Executor"
echo "[MIN=$MIN]"
echo "[MAX=$MAX]"
echo "[SIZE=$SIZE]"
echo
echo "[MIN_MODIF=$MIN_MODIF]"
echo "[MAX_MODIF=$MAX_MODIF]"
echo "[TURNS_MIN=$TURNS_MIN]"
echo "[TURNS_MAX=$TURNS_MAX]"
# Create a results file with the previous discovered passwords
#  to avoid re-calculating their salts again
cp -f results john.pot

# Unpack the runtime
echo "Unpacking runtime"
tar -xzf runtime.tgz 2>&1

# Add configuration option, the filter, with the right values:
cat >> john.conf <<EOF
[List.External:Block]
/*
 * This creates a virtual block in the password space. Specifying the min and max values,
 * it will only check those passwords whose order number is in between. It is useful for
 * distributed computing, to divide the whole space into blocks and compute each
 * separately. The only way found so far to make John exit after finishing is to make it
 * generate a DBZ exception (for wordlist, simple and incremental modes).
 * You can also use a custom external block, if you define the same margins and operations.
 */

int min, max;				// Search margins
int turns_min, turns_max;		// Number of times we go past 2147483648 (unsig_int)
int min_secure, max_secure;   		// Security margins (to avoid losing passwords)
int number;                             // Current word number

void init()
{
        min = $MIN_MODIF; 
        max = $MAX_MODIF;     		// Doesn't matter if any turns around max_int
	turns_min = $TURNS_MIN;       
	turns_max = $TURNS_MAX;       
        number = 0;
        min_secure = min - 1;           // Include the edges
        max_secure = max + 1;         
}

void filter()
{
	if (++number < 0) { number = 0; turns_min--; turns_max--; }   // Decrease turns if number flipped. 
	if (turns_min > 0) { word = 0; return; }  // Skip until turns_int==0, the right turn
        if (turns_min == 0 && number < min_secure) { word = 0; return; }  // Advance until the first word
        if (turns_max > 0) { return; }  // Scan the word until we reach the right max turn 
        if (number > max_secure) { event_pending = event_abort = 1; }  // Exit
}
EOF

# Initialize running time
TIME=$(date +%s)

# Execute john
echo "Starting john"
./john -i --external=Block passwd 

# Capture exit value
EXITVAL=$?
echo "[EXITVAL=$EXITVAL]"

# Count time
let RUNTIME=$(date +%s)-$TIME
echo "[RUNTIME=$RUNTIME]"

# Remove previously discovered lines
touch tmp_results
cat john.pot | while read line; do
    grep -F "$line" results > /dev/null
    if [ $? != 0 ]; then
      # only print the line if not in results
      echo $line >> tmp_results
    fi
  done
mv -f tmp_results john.pot

# And exit
exit $EXITVAL

